/*
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "Config.h"
#ifdef _TIMIDITY

#include <iostream>

#include "GUI/GLPane.h"
#include "Midi/Sequence.h"
#include "Midi/CommonMidiUtils.h"

#include "wx/process.h"

namespace AriaMaestosa {
	

class MyProcess : public wxProcess
{
	
	bool running;
	
public:
	
MyProcess() : wxProcess(wxPROCESS_DEFAULT)
{
	running=false;
}
	
void startingProcess()
{
	running = true;
}

void OnTerminate(int pid, int status)
{
	std::cout << "Done!" << std::endl;
	running = false;
}


void stop()
{
	if(!running) return;
	std::cout << "stop" << std::endl;
	
	wxKillError error;
	if(! wxKill( MyProcess::GetPid(), wxSIGTERM, &error ) )
	{
		std::cout << "error stopping timidity: " << error << std::endl;	
		if(error == wxKILL_BAD_SIGNAL) std::cout << "** no such signal" << std::endl;
		if(error == wxKILL_ACCESS_DENIED) std::cout << "** permission denied" << std::endl;
		if(error == wxKILL_NO_PROCESS) std::cout << "** no such process" << std::endl;
		if(error == wxKILL_ERROR) std::cout << "** unspecified error" << std::endl;
	}
	running = false;
}
	
bool isRunning()
{
	return running;	
}

};
	
	namespace PlatformMidiManager {
		
// FIXME - what's the difference between trackPlaybackProgression() and getCurrentTick() ???

/*
	bool playSequence(Sequence* sequence, int* startTick);
	bool playSelected(Sequence* sequence, int* startTick);
	void exportAudioFile(Sequence* sequence, char* filepath);
	bool exportMidiFile(Sequence* sequence, char* filepath);
	
	// returns current midi tick, or -1 if over
	int trackPlaybackProgression();
	
	bool isPlaying();
	
    void stop();
	
	// midi tick currently being played, -1 if none
    int getCurrentTick();
    	
	// called when app opens
    void initMidiPlayer();
	
	// called when app closes
    void freeMidiPlayer();
 
	// play/stop a single preview note
	void playNote(int noteNum, int volume, int duration, int channel, int instrument);
	void stopNote();
	
	const char* getAudioExtension();
	const char* getAudioWildcard();
*/
	    MyProcess* process = NULL;
	
		Sequence* sequence;
		bool playing=false;

	bool playSequence(Sequence* sequence, int* startTick)
        {
                AriaMaestosa::exportMidiFile(sequence, "temp.mid");
		process->startingProcess();
		wxExecute( wxString("timidity ") + wxString("temp.mid"), wxEXEC_ASYNC, process );
		playing = true;
                *startTick = 0;
        }

	bool playSelected(Sequence* sequence, int* startTick)
        {
        }

	void exportAudioFile(Sequence* sequence, char* filepath)
        {
        }

	bool exportMidiFile(Sequence* sequence, char* filepath)
        {
           AriaMaestosa::exportMidiFile(sequence, filepath);
        }
	
	// returns current midi tick, or -1 if over
	int trackPlaybackProgression()
        {
          return -1;
        }
	
	bool isPlaying()
        {
          return playing;
        }
	
		
                const char* getAudioExtension()
                {
	                return ".wav";	
                }
                const char* getAudioWildcard()
                {
	                return wxGetTranslation("WAV file|*.wav");	
                }


		void playNote(int noteNum, int volume, int duration, int channel, int instrument)
		{
			if(playing) return;
		}
		
		void stopNote()
		{	
		}
		
		void stop()
		{
			process->stop();
			playing = false;
		}
		
		int getCurrentTick()
		{
			if(process->isRunning()) return 0;
			return -1;
		}
		
		
		void initMidiPlayer()
		{
			process = new MyProcess();
		}
		
		void freeMidiPlayer()
		{
		}
		
	}
}

#endif
